@@ -124,12 +124,20 @@ class TeX:
             self.main = text
             self.postamble = ""
 
+        self.original = text
+
     def get(self):
         """
         Return document.
         """
         return self.preamble + self.start + self.main + self.postamble
 
+    def changed(self):
+        """
+        Check if the document has changed.
+        """
+        return self.original != self.get()
+
     def float_filenames(self, cmd: str = r"\includegraphics") -> list[tuple[str]]:
         r"""
         Extract the keys of 'float' commands
@@ -163,6 +171,10 @@ class TeX:
 
             raise OSError(f"Cannot find {name:s}")
 
+        tmp = self.main
+        self.remove_commentlines()
+        self.remove_comments()
+
         # read the contents of the command
         # - "\includegraphics" accepts "\includegraphics[...]{...}"
         # - "\bibliography" rejects "\bibliographystyle{...}"
@@ -171,6 +183,8 @@ class TeX:
             if i[0] in ["[", "{"]:
                 include += [i.split("{")[1].split("}")[0]]
 
+        self.main = tmp
+
         # extract the filename
         out = [(i, filename(self.dirname, i)) for i in include]
 
@@ -212,22 +226,15 @@ class TeX:
         :return: Unique list of keys in the order or appearance.
         """
 
-        # extract keys from "cite"
-        def extract(string):
-            try:
-                match = r"([pt])?(\[.*\]\[.*\])?(\{[a-zA-Z0-9\.\,\-\ \_]*\})"
-                return list(re.split(match, string)[3][1:-1].split(","))
-            except IndexError:
-                if len(string) >= 100:
-                    string = string[:100]
-                raise OSError(f"Error in interpreting\n {string:s} ...")
+        curly_braces = find_matching(self.main, "{", "}", ignore_escaped=True)
+        cite = []
 
-        # read all keys in "cite", "citet", "citep" commands
-        cite = [extract(i) for i in self.main.split(r"\cite")[1:]]
-        cite = list({item for sublist in cite for item in sublist})
-        cite = [i.replace(" ", "") for i in cite]
+        for i in re.finditer(r"(\\cite)([pt])?(\[.*\]\[.*\])?(\{)", self.main):
+            o = i.span()[1]
+            c = curly_braces[o - 1]
+            cite += list(filter(None, self.main[o:c].replace("\n", " ").split(",")))
 
-        return cite
+        return [i.replace(" ", "") for i in cite]
 
     def find_by_extension(self, ext: str) -> list[str]:
         r"""
@@ -273,7 +280,10 @@ class TeX:
         Remove comments form the main text.
         """
 
-        self.main = re.sub(r"([^%]*)(.*)(\n)", r"\1\3", self.main)
+        tmp = self.main.split("\n")
+        for i in range(len(tmp)):
+            tmp[i] = re.sub(r"([^%]*)(%)(.*)$", r"\1", tmp[i])
+        self.main = "\n".join(tmp)
 
     def _replace_command_impl(self, cmd: str, nargs: int, replace: str):
         """
@@ -385,17 +395,22 @@ class TeX:
 
         self._replace_command_impl(cmd, nargs, replace)
 
-    def change_label(self, old_label: str, new_label: str):
+    def change_label(self, old_label: str, new_label: str, overwrite: bool = False):
         r"""
         Change label in ``\label{...}`` and ``\ref{...}`` (-like) commands.
 
         :param old_label: Old label.
         :param new_label: New label.
+        :param overwrite: Overwrite existing labels.
         """
 
         if old_label == new_label:
             return
 
+        if not overwrite:
+            if new_label in self.labels():
+                raise OSError(f'Label "{new_label:s}" already exists')
+
         old = re.escape(old_label)
         new = new_label
 
@@ -752,7 +767,7 @@ def texcleanup(args: list[str]):
             tex.remove_commentlines()
 
         if args.remove_comments:
-            tex.remove_commentlines()
+            tex.remove_comments()
 
         if args.replace_command:
             for i in args.replace_command:
@@ -768,8 +783,9 @@ def texcleanup(args: list[str]):
         if args.use_cleveref:
             tex.use_cleveref()
 
-        with open(file, "w") as file:
-            file.write(tex.get())
+        if tex.changed():
+            with open(file, "w") as file:
+                file.write(tex.get())
 
 
 def _texcleanup_catch():
@@ -787,6 +803,7 @@ def _texplain_parser():
 
     desc = "Create a clean output directory with only included files/citations."
     parser = argparse.ArgumentParser(description=desc)
+    parser.add_argument("-c", "--keep-comments", action="store_true", help="Keep comments")
     parser.add_argument("-v", "--version", action="version", version=version)
     parser.add_argument("input", type=str, help="TeX file")
     parser.add_argument("outdir", type=str, help="Output directory")
@@ -814,6 +831,10 @@ def texplain(args: list[str]):
     new = deepcopy(old)
     new.dirname = args.outdir
 
+    if not args.keep_comments:
+        new.remove_commentlines()
+        new.remove_comments()
+
     includegraphics = old.float_filenames(r"\includegraphics")
     bibfiles = old.float_filenames(r"\bibliography")
     bibkeys = old.citation_keys()
